@@ -13,13 +13,14 @@
 #include <kaboutdata.h>
 #include <KConfig>
 #include <KConfigGroup>
+#include <kcrash.h>
 #include <klocalizedstring.h>
 #include <KSharedConfig>
 
 #include <QFile>
 #include <QCommandLineParser>
 #include <QApplication>
- 
+
 #include "ControlView.h"
 #include "KdeMainWindow.h"
 #include "GeoUriParser.h"
@@ -43,7 +44,7 @@ int main ( int argc, char *argv[] )
                           ControlView::applicationVersion(),
                           i18n( "A World Atlas." ),
                           KAboutLicense::LGPL_V2,
-                          i18n( "(c) 2007-%1" ).arg( 2015 ),
+                          i18n( "(c) 2007-%1", QLatin1String("2015") ),
                           QString(),
                           "http://marble.kde.org/" );
 
@@ -264,6 +265,9 @@ int main ( int argc, char *argv[] )
                                 " Marble's predecessor \"Kartographer\"." ));
     KAboutData::setApplicationData(aboutData);
 
+    KCrash::setCrashHandler( KCrash::defaultCrashHandler );
+    KCrash::setDrKonqiEnabled( true );
+
     QApplication app( argc, argv );
     QCommandLineParser parser;
     parser.addHelpOption();
