@@ -272,7 +272,7 @@ bool MarblePart::openFile()
     }
 
     foreach( const QString &fileName, fileNames ) {
-        openUrl( fileName );
+        openUrl( QUrl::fromLocalFile(fileName) );
     }
 
     return true;
@@ -1346,7 +1346,7 @@ void MarblePart::setupDownloadProgressBar()
     HttpDownloadManager * const downloadManager =
         m_controlView->marbleModel()->downloadManager();
     Q_ASSERT( downloadManager );
-    connect( downloadManager, SIGNAL(progressChanged( int, int )), SLOT(handleProgress( int, int )) );
+    connect( downloadManager, SIGNAL(progressChanged(int,int)), SLOT(handleProgress(int,int)) );
     connect( downloadManager, SIGNAL(jobRemoved()), SLOT(removeProgressItem()) );
 }
 
