@@ -97,7 +97,9 @@ MainWindow::MainWindow( const QString &marbleDataPath, const QVariantMap &cmdLin
 
     // prevent triggering of network requests under Maemo, presumably due to qrc: URLs
     m_networkAccessManager.setNetworkAccessible( QNetworkAccessManager::NotAccessible );
+#ifndef MARBLE_NO_WEBKIT
     m_legendBrowser->page()->setNetworkAccessManager( &m_networkAccessManager );
+#endif
 
     QString selectedPath = marbleDataPath.isEmpty() ? readMarbleDataPath() : marbleDataPath;
     if ( !selectedPath.isEmpty() )
@@ -707,4 +709,4 @@ void MainWindow::fallBackToDefaultTheme()
     }
 }
 
-#include "MobileMainWindow.moc"
+#include "moc_MobileMainWindow.cpp"
