@@ -59,7 +59,7 @@ public:
     void addGeoDataFile( const QString &fileName );
 
 protected:
-    void  closeEvent(QCloseEvent *event);
+    void  closeEvent( QCloseEvent *event );
 
 private:
     void  createActions();
@@ -123,6 +123,7 @@ private Q_SLOTS:
     void  enterWhatsThis();
     void  aboutMarble();
     void  handbook();
+    void  openForum();
 
     //Bookmark Menu
     void  openEditBookmarkDialog();
@@ -138,6 +139,7 @@ private Q_SLOTS:
     void  downloadRegion();
 
     void showZoomLevel( bool show );
+    void changeAngleDisplayUnit( QAction *action );
 
     void handleProgress( int, int );
     void removeProgressItem();
@@ -201,6 +203,7 @@ private:
     QAction *m_aboutQtAction;
     QAction *m_lockFloatItemsAction;
     QAction *m_handbookAction;
+    QAction *m_forumAction;
 
     // Status Bar
     QString     m_position;
@@ -213,6 +216,10 @@ private:
     QLabel      *m_clockLabel;
     QProgressBar *m_downloadProgressBar;
     QAction     *m_toggleTileLevelAction;
+    QActionGroup *m_angleDisplayUnitActionGroup;
+    QAction     *m_dmsDegreeAction;
+    QAction     *m_decimalDegreeAction;
+    QAction     *m_utmAction;
     void updateStatusBar();
 
     //Bookmark Menu
