@@ -251,8 +251,7 @@ GeoDataPlacemark EditBookmarkDialog::bookmark() const
     GeoDataPlacemark bookmark;
     bookmark.setName( name() );
     bookmark.setDescription( description() );
-    GeoDataStyle *newStyle = new GeoDataStyle( *bookmark.style() );
-    newStyle->iconStyle().setIcon( QImage() );
+    GeoDataStyle::Ptr newStyle(new GeoDataStyle( *bookmark.style() ));
     newStyle->iconStyle().setIconPath( iconLink() );
     bookmark.setStyle( newStyle );
     //allow for HTML in the description
@@ -308,4 +307,4 @@ QString EditBookmarkDialog::iconLink() const
 
 }
 
-#include "EditBookmarkDialog.moc"
+#include "moc_EditBookmarkDialog.cpp"
