@@ -53,6 +53,8 @@ class MARBLE_EXPORT EditBookmarkDialog : public QDialog
 
     qreal range() const;
 
+    QString iconLink() const;
+
     void setName( const QString &name );
     void setReverseGeocodeName();
 
@@ -64,6 +66,8 @@ class MARBLE_EXPORT EditBookmarkDialog : public QDialog
 
     void setFolderName( const QString &name );
 
+    void setIconLink( const QString &iconLink );
+
 
  private:
     Q_PRIVATE_SLOT( d, void openNewFolderDialog() )
