@@ -13,7 +13,7 @@
 #include "GeoSceneLayer.h"
 #include "GeoSceneMap.h"
 #include "GeoSceneDocument.h"
-#include "GeoSceneTextureTile.h"
+#include "GeoSceneTextureTileDataset.h"
 #include "HttpDownloadManager.h"
 #include "Tile.h"
 #include "TileLoader.h"
@@ -21,6 +21,7 @@
 #include "MarbleDebug.h"
 #include "MapThemeManager.h"
 #include "TileId.h"
+#include "PluginManager.h"
 
 #include <QLabel>
 #include <qmath.h>
@@ -31,32 +32,38 @@ namespace Marble
 class ElevationModelPrivate
 {
 public:
-    ElevationModelPrivate( ElevationModel *_q, HttpDownloadManager *downloadManager )
+    ElevationModelPrivate( ElevationModel *_q, HttpDownloadManager *downloadManager, PluginManager* pluginManager )
         : q( _q ),
-          m_tileLoader( downloadManager, 0 ),
-          m_textureLayer( 0 )
+          m_tileLoader( downloadManager, pluginManager ),
+          m_textureLayer( 0 ),
+          m_srtmTheme(0)
     {
         m_cache.setMaxCost( 10 ); //keep 10 tiles in memory (~17MB)
 
-        const GeoSceneDocument *srtmTheme = MapThemeManager::loadMapTheme( "earth/srtm2/srtm2.dgml" );
-        if ( !srtmTheme ) {
+        m_srtmTheme = MapThemeManager::loadMapTheme( "earth/srtm2/srtm2.dgml" );
+        if ( !m_srtmTheme ) {
             mDebug() << "Failed to load map theme earth/srtm2/srtm2.dgml. Check your installation. No elevation will be returned.";
             return;
         }
 
-        const GeoSceneHead *head = srtmTheme->head();
+        const GeoSceneHead *head = m_srtmTheme->head();
         Q_ASSERT( head );
 
-        const GeoSceneMap *map = srtmTheme->map();
+        const GeoSceneMap *map = m_srtmTheme->map();
         Q_ASSERT( map );
 
         const GeoSceneLayer *sceneLayer = map->layer( head->theme() );
         Q_ASSERT( sceneLayer );
 
-        m_textureLayer = dynamic_cast<GeoSceneTextureTile*>( sceneLayer->datasets().first() );
+        m_textureLayer = dynamic_cast<GeoSceneTextureTileDataset*>( sceneLayer->datasets().first() );
         Q_ASSERT( m_textureLayer );
     }
 
+    ~ElevationModelPrivate()
+    {
+       delete m_srtmTheme;
+    }
+
     void tileCompleted( const TileId & tileId, const QImage &image )
     {
         m_cache.insert( tileId, new QImage( image ) );
@@ -67,18 +74,24 @@ public:
     ElevationModel *q;
 
     TileLoader m_tileLoader;
-    const GeoSceneTextureTile *m_textureLayer;
+    const GeoSceneTextureTileDataset *m_textureLayer;
     QCache<TileId, const QImage> m_cache;
+    GeoSceneDocument *m_srtmTheme;
 };
 
-ElevationModel::ElevationModel( HttpDownloadManager *downloadManager, QObject *parent ) :
+ElevationModel::ElevationModel( HttpDownloadManager *downloadManager, PluginManager* pluginManager, QObject *parent ) :
     QObject( parent ),
-    d( new ElevationModelPrivate( this, downloadManager ) )
+    d( new ElevationModelPrivate( this, downloadManager, pluginManager ) )
 {
     connect( &d->m_tileLoader, SIGNAL(tileCompleted(TileId,QImage)),
              this, SLOT(tileCompleted(TileId,QImage)) );
 }
 
+ElevationModel::~ElevationModel()
+{
+    delete d;
+}
+
 
 qreal ElevationModel::height( qreal lon, qreal lat ) const
 {
