@@ -28,11 +28,6 @@
 namespace Marble
 {
 
-bool zValueLessThan( GeoGraphicsItem* i1, GeoGraphicsItem* i2 )
-{
-    return i1->zValue() < i2->zValue();
-}
-
 class GeoGraphicsScenePrivate
 {
 public:
@@ -47,32 +42,30 @@ public:
         q->clear();
     }
 
-    static void mergeItems( QList<GeoGraphicsItem *> &result, const QList<GeoGraphicsItem *> &objects, int maxZoomLevel );
-
     QMap<TileId, QList<GeoGraphicsItem*> > m_items;
     QMultiHash<const GeoDataFeature*, TileId> m_features;
 
     // Stores the items which have been clicked;
     QList<GeoGraphicsItem*> m_selectedItems;
 
-    GeoDataStyle *highlightStyle(const GeoDataDocument *document, const GeoDataStyleMap &styleMap);
+    GeoDataStyle::Ptr highlightStyle(const GeoDataDocument *document, const GeoDataStyleMap &styleMap);
 
     void selectItem( GeoGraphicsItem *item );
-    void applyHighlightStyle( GeoGraphicsItem *item, GeoDataStyle *style );
+    void applyHighlightStyle(GeoGraphicsItem *item, const GeoDataStyle::Ptr &style );
 };
 
-GeoDataStyle *GeoGraphicsScenePrivate::highlightStyle( const GeoDataDocument *document,
+GeoDataStyle::Ptr GeoGraphicsScenePrivate::highlightStyle( const GeoDataDocument *document,
                                                        const GeoDataStyleMap &styleMap )
 {
     // @todo Consider QUrl parsing when external styles are suppported
     QString highlightStyleId = styleMap.value("highlight");
     highlightStyleId.remove('#');
     if ( !highlightStyleId.isEmpty() ) {
-        GeoDataStyle *highlightStyle = new GeoDataStyle( document->style(highlightStyleId) );
+        GeoDataStyle::Ptr highlightStyle(new GeoDataStyle( *document->style(highlightStyleId) ));
         return highlightStyle;
     }
     else {
-        return 0;
+        return GeoDataStyle::Ptr();
     }
 }
 
@@ -81,7 +74,7 @@ void GeoGraphicsScenePrivate::selectItem( GeoGraphicsItem* item )
     m_selectedItems.append( item );
 }
 
-void GeoGraphicsScenePrivate::applyHighlightStyle(GeoGraphicsItem* item, GeoDataStyle* highlightStyle )
+void GeoGraphicsScenePrivate::applyHighlightStyle(GeoGraphicsItem* item, const GeoDataStyle::Ptr &highlightStyle )
 {
     item->setHighlightStyle( highlightStyle );
     item->setHighlighted( true );
@@ -148,10 +141,15 @@ QList< GeoGraphicsItem* > GeoGraphicsScene::items( const GeoDataLatLonBox &box,
         for ( int x = x1; x <= x2; ++x ) {
             for ( int y = y1; y <= y2; ++y ) {
                 const TileId tileId = TileId( 0, level, x, y );
-                GeoGraphicsScenePrivate::mergeItems( result, d->m_items.value( tileId ), zoomLevel );
+                foreach(GeoGraphicsItem *object, d->m_items.value( tileId )) {
+                    if (object->minZoomLevel() <= zoomLevel && object->visible()) {
+                        result.push_back(object);
+                    }
+                }
             }
         }
     }
+
     return result;
 }
 
@@ -192,7 +190,7 @@ void GeoGraphicsScene::applyHighlight( const QVector< GeoDataPlacemark* > &selec
                             styleUrl.remove('#');
                             if ( !styleUrl.isEmpty() ) {
                                 GeoDataStyleMap const &styleMap = doc->styleMap( styleUrl );
-                                GeoDataStyle *style = d->highlightStyle( doc, styleMap );
+                                GeoDataStyle::Ptr style = d->highlightStyle( doc, styleMap );
                                 if ( style ) {
                                     d->selectItem( item );
                                     d->applyHighlightStyle( item, style );
@@ -207,7 +205,7 @@ void GeoGraphicsScene::applyHighlight( const QVector< GeoDataPlacemark* > &selec
                             */
                             else {
                                 foreach ( const GeoDataStyleMap &styleMap, doc->styleMaps() ) {
-                                    GeoDataStyle *style = d->highlightStyle( doc, styleMap );
+                                    GeoDataStyle::Ptr style = d->highlightStyle( doc, styleMap );
                                     if ( style ) {
                                         d->selectItem( item );
                                         d->applyHighlightStyle( item, style );
@@ -265,29 +263,11 @@ void GeoGraphicsScene::addItem( GeoGraphicsItem* item )
     const TileId key = TileId::fromCoordinates( GeoDataCoordinates(west, north, 0), zoomLevel ); // same as GeoDataCoordinates(east, south, 0), see above
 
     QList< GeoGraphicsItem* >& tileList = d->m_items[key];
-    QList< GeoGraphicsItem* >::iterator position = qLowerBound( tileList.begin(), tileList.end(), item, zValueLessThan );
+    QList< GeoGraphicsItem* >::iterator position = qLowerBound( tileList.begin(), tileList.end(), item, GeoGraphicsItem::zValueLessThan );
     tileList.insert( position, item );
     d->m_features.insert( item->feature(), key );
 }
 
-void GeoGraphicsScenePrivate::mergeItems( QList<GeoGraphicsItem *> &result, const QList<GeoGraphicsItem *> &objects, int maxZoomLevel )
-{
-    QList< GeoGraphicsItem* >::iterator before = result.begin();
-    QList< GeoGraphicsItem* >::const_iterator currentItem = objects.constBegin();
-    while( currentItem != objects.end() ) {
-        while( ( currentItem != objects.end() )
-          && ( ( before == result.end() ) || ( (*currentItem)->zValue() < (*before)->zValue() ) ) ) {
-            if( (*currentItem)->minZoomLevel() <= maxZoomLevel && (*currentItem)->visible() ) {
-                before = result.insert( before, *currentItem );
-            }
-            ++currentItem;
-        }
-        if ( before != result.end() ) {
-            ++before;
-        }
-    }
 }
 
-}
-
-#include "GeoGraphicsScene.moc"
+#include "moc_GeoGraphicsScene.cpp"
