@@ -14,6 +14,7 @@
 #include <QList>
 #include <QPainterPath>
 #include <QRegion>
+#include <qmath.h>
 
 #include "MarbleDebug.h"
 
@@ -31,10 +32,11 @@
 
 using namespace Marble;
 
-GeoPainterPrivate::GeoPainterPrivate( const ViewportParams *viewport, MapQuality mapQuality )
+GeoPainterPrivate::GeoPainterPrivate( GeoPainter* q, const ViewportParams *viewport, MapQuality mapQuality )
         : m_viewport( viewport ),
         m_mapQuality( mapQuality ),
-        m_x( new qreal[100] )
+        m_x( new qreal[100] ),
+        m_parent(q)
 {
 }
 
@@ -174,14 +176,33 @@ bool GeoPainterPrivate::doClip( const ViewportParams *viewport )
     return ( radius > viewport->width() / 2 || radius > viewport->height() / 2 );
 }
 
+qreal GeoPainterPrivate::normalizeAngle(qreal angle)
+{
+    angle = fmodf(angle, 360);
+    return angle < 0 ? angle + 360 : angle;
+}
+
+void GeoPainterPrivate::drawTextRotated( const QPointF &startPoint, qreal angle, const QString &text )
+{
+    QRectF textRect(startPoint, m_parent->fontMetrics().size( 0, text));
+    QTransform const oldTransform = m_parent->transform();
+    m_parent->translate(startPoint);
+    m_parent->rotate(angle);
+    m_parent->translate( -startPoint - QPointF(0.0, m_parent->fontMetrics().height()/2.0)  );
+
+    m_parent->drawText( textRect, text);
+    m_parent->setTransform(oldTransform);
+}
+
 // -------------------------------------------------------------------------------------------------
 
 GeoPainter::GeoPainter( QPaintDevice* pd, const ViewportParams *viewport, MapQuality mapQuality )
     : ClipPainter( pd, GeoPainterPrivate::doClip( viewport ) ),
-      d( new GeoPainterPrivate( viewport, mapQuality ) )
+      d( new GeoPainterPrivate( this, viewport, mapQuality ) )
 {
     const bool antialiased = mapQuality == HighQuality || mapQuality == PrintQuality;
     setRenderHint( QPainter::Antialiasing, antialiased );
+    ClipPainter::setScreenClip(false);
 }
 
 
@@ -485,7 +506,8 @@ void GeoPainter::drawPixmap ( const GeoDataCoordinates & centerPosition,
 
 void GeoPainter::drawPolyline ( const GeoDataLineString & lineString,
                                 const QString& labelText,
-                                LabelPositionFlags labelPositionFlags )
+                                LabelPositionFlags labelPositionFlags,
+                                const QColor& labelColor,const QFont& labelFont)
 {
     // Immediately leave this method now if:
     // - the object is not visible in the viewport or if
@@ -506,7 +528,88 @@ void GeoPainter::drawPolyline ( const GeoDataLineString & lineString,
             ClipPainter::drawPolyline( *itPolygon );
         }
     }
-    else {
+    else if ( labelPositionFlags.testFlag( FollowLine ) ) {
+        const qreal maximumLabelFontSize = 20;
+        qreal fontSize = pen().widthF() * 0.45;
+        fontSize = qMin( fontSize, maximumLabelFontSize );
+
+        QVector<QPointF> labelNodes;
+        QRectF viewportRect = QRectF(QPointF(0, 0), d->m_viewport->size());
+        foreach( QPolygonF* itPolygon, polygons ) {
+            if (!itPolygon->boundingRect().intersects(viewportRect)) {
+                continue;
+            }
+
+            labelNodes.clear();
+            ClipPainter::drawPolyline( *itPolygon, labelNodes, labelPositionFlags );
+
+            save();
+
+            if (fontSize >= 6.0) {
+                QFont font = labelFont;
+                font.setPointSizeF(fontSize);
+                setFont(font);
+                int labelWidth = fontMetrics().width( labelText );
+
+                QPainterPath path;
+                path.addPolygon(*itPolygon);
+                qreal pathLength = path.length();
+                if (pathLength == 0) continue;
+
+                int maxNumLabels = static_cast<int>(pathLength / labelWidth);
+
+                if (maxNumLabels > 0) {
+                    qreal textRelativeLength = labelWidth / pathLength;
+                    int numLabels = 1;
+                    if (maxNumLabels > 1) {
+                        numLabels = maxNumLabels/2;
+                    }
+                    qreal offset = (1.0 - numLabels*textRelativeLength)/numLabels;
+                    qreal startPercent = offset/2.0;
+
+                    setPen(labelColor);
+
+                    for (int k = 0; k < numLabels; ++k, startPercent += textRelativeLength + offset) {
+                        QPointF point = path.pointAtPercent(startPercent);
+                        QPointF endPoint = path.pointAtPercent(startPercent + textRelativeLength);
+
+                        if ( viewport().contains(point.toPoint()) || viewport().contains(endPoint.toPoint()) ) {
+                            qreal angle = -path.angleAtPercent(startPercent);
+                            qreal angle2 = -path.angleAtPercent(startPercent + textRelativeLength);
+                            angle = GeoPainterPrivate::normalizeAngle(angle);
+                            angle2 = GeoPainterPrivate::normalizeAngle(angle2);
+                            bool upsideDown = angle > 90.0 && angle < 270.0;
+
+                            if ( qAbs(angle - angle2) < 3.0 ) {
+                                if ( upsideDown ) {
+                                    angle += 180.0;
+                                    point = path.pointAtPercent(startPercent + textRelativeLength);
+                                }
+
+                                d->drawTextRotated(point, angle, labelText);
+                            } else {
+                                for (int i = 0; i < labelText.length(); ++i) {
+                                    qreal currentGlyphTextLength = fontMetrics().width(labelText.left(i)) / pathLength;
+
+                                    if ( !upsideDown ) {
+                                        angle = -path.angleAtPercent(startPercent + currentGlyphTextLength);
+                                        point = path.pointAtPercent(startPercent + currentGlyphTextLength);
+                                    }
+                                    else {
+                                        angle = -path.angleAtPercent(startPercent + textRelativeLength - currentGlyphTextLength) + 180;
+                                        point = path.pointAtPercent(startPercent + textRelativeLength - currentGlyphTextLength);
+                                    }
+
+                                    d->drawTextRotated(point, angle, labelText.at(i));
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            restore();
+        }
+    } else {
         int labelWidth = fontMetrics().width( labelText );
         int labelAscent = fontMetrics().ascent();
 
@@ -514,7 +617,9 @@ void GeoPainter::drawPolyline ( const GeoDataLineString & lineString,
         foreach( QPolygonF* itPolygon, polygons ) {
             labelNodes.clear();
             ClipPainter::drawPolyline( *itPolygon, labelNodes, labelPositionFlags );
-            if ( !labelNodes.isEmpty() ) {
+            if ( !labelNodes.isEmpty() && labelColor != Qt::transparent ) {
+                QPen const oldPen = pen();
+                setPen(labelColor);
                 foreach ( const QPointF& labelNode, labelNodes ) {
                     QPointF labelPosition = labelNode + QPointF( 3.0, -2.0 );
 
@@ -528,6 +633,7 @@ void GeoPainter::drawPolyline ( const GeoDataLineString & lineString,
 
                     drawText( QRectF( labelPosition, fontMetrics().size( 0, labelText) ), labelText );
                 }
+                setPen(oldPen);
             }
         }
     }
@@ -651,61 +757,76 @@ void GeoPainter::drawPolygon ( const GeoDataPolygon & polygon,
     }
     // mDebug() << "Drawing Polygon";
 
-    // Creating the outer screen polygons first
     QVector<QPolygonF*> outerPolygons;
+    QVector<QPolygonF*> innerPolygons;
     d->m_viewport->screenCoordinates( polygon.outerBoundary(), outerPolygons );
 
-    // Now creating the "holes" by cutting away the inner boundaries:
-
-    // In QPathClipper We Trust ...
-    // ... and in the speed of a threesome of nested foreachs!
-
-    QVector<QPolygonF> outline;
     QPen const oldPen = pen();
+
     // When inner boundaries exist, the outline of the polygon must be painted
     // separately to avoid connections between the outer and inner boundaries
     // To avoid performance penalties the separate painting is only done when
     // it's really needed. See review 105019 for details.
-    bool const needOutlineWorkaround = !polygon.innerBoundaries().isEmpty();
-    if ( needOutlineWorkaround ) {
-        foreach( QPolygonF* polygon, outerPolygons ) {
-            outline << *polygon;
-        }
-        setPen( QPen( Qt::NoPen ) );
-    }
+    bool const hasInnerBoundaries = !polygon.innerBoundaries().isEmpty();
+    bool innerBoundariesOnScreen = false;
 
+    if ( hasInnerBoundaries ) {
+        QVector<GeoDataLinearRing> innerBoundaries = polygon.innerBoundaries();
 
-    QVector<GeoDataLinearRing> innerBoundaries = polygon.innerBoundaries(); 
-    foreach( const GeoDataLinearRing& itInnerBoundary, innerBoundaries ) {
-        QVector<QPolygonF*> innerPolygons;
-        d->m_viewport->screenCoordinates( itInnerBoundary, innerPolygons );
-
-        if ( needOutlineWorkaround ) {
-            foreach( QPolygonF* polygon, innerPolygons ) {
-                outline << *polygon;
+        const GeoDataLatLonAltBox & viewLatLonAltBox = d->m_viewport->viewLatLonAltBox();
+        foreach( const GeoDataLinearRing& itInnerBoundary, innerBoundaries ) {
+            if ( viewLatLonAltBox.intersects(itInnerBoundary.latLonAltBox())
+                 && d->m_viewport->resolves(itInnerBoundary.latLonAltBox()) )  {
+                innerBoundariesOnScreen = true;
+                break;
             }
         }
 
-        foreach( QPolygonF* itOuterPolygon, outerPolygons ) {
-            foreach( QPolygonF* itInnerPolygon, innerPolygons ) {
-                *itOuterPolygon = itOuterPolygon->subtracted( *itInnerPolygon );
+        if (innerBoundariesOnScreen) {
+            // Cut the outer polygons to the viewport
+            QVector<QPointF> viewportPolygon = QPolygonF(QRectF(0, 0, d->m_viewport->width(), d->m_viewport->width()));
+            foreach(QPolygonF* outerPolygon, outerPolygons) {
+                *outerPolygon = outerPolygon->intersected(QPolygonF(viewportPolygon));
+            }
+
+            setPen( QPen( Qt::NoPen ) );
+
+            // Create the inner screen polygons
+            foreach( const GeoDataLinearRing& itInnerBoundary, innerBoundaries ) {
+                QVector<QPolygonF*> innerPolygonsPerBoundary;
+
+                d->m_viewport->screenCoordinates( itInnerBoundary, innerPolygonsPerBoundary );
+
+                foreach( QPolygonF* innerPolygonPerBoundary, innerPolygonsPerBoundary ) {
+                    innerPolygons << innerPolygonPerBoundary;
+                }
             }
         }
-        qDeleteAll( innerPolygons );    
     }
 
-    foreach( QPolygonF* itOuterPolygon, outerPolygons ) {
-        ClipPainter::drawPolygon( *itOuterPolygon, fillRule );
+    foreach( QPolygonF* outerPolygon, outerPolygons ) {
+        if (hasInnerBoundaries && innerBoundariesOnScreen) {
+            QRegion clip(outerPolygon->toPolygon());
+
+            foreach(QPolygonF* innerPolygon, innerPolygons) {
+                clip-=QRegion(innerPolygon->toPolygon());
+            }
+            ClipPainter::setClipRegion(clip);
+        }
+        ClipPainter::drawPolygon( *outerPolygon, fillRule );
     }
 
-    if ( needOutlineWorkaround ) {
+    if ( hasInnerBoundaries && innerBoundariesOnScreen ) {
         setPen( oldPen );
-        foreach( const QPolygonF &polygon, outline ) {
-            ClipPainter::drawPolyline( polygon );
+        foreach( const QPolygonF* outerPolygon, outerPolygons ) {
+            ClipPainter::drawPolyline( *outerPolygon );
+        }
+        foreach( const QPolygonF* innerPolygon, innerPolygons ) {
+            ClipPainter::drawPolyline( *innerPolygon );
         }
     }
-
-    qDeleteAll( outerPolygons );    
+    qDeleteAll(outerPolygons);
+    qDeleteAll(innerPolygons);
 }
 
 
